<?php

namespace Drupal\subgroup\Plugin\Group\RelationHandler;

use Drupal\Core\Session\AccountInterface;
use Drupal\group\Entity\GroupInterface;
use Drupal\group\Plugin\Group\RelationHandler\AccessControlInterface;
use Drupal\group\Plugin\Group\RelationHandler\AccessControlTrait;

/**
 * Provides access control for subgroup GroupContent entities.
 */
class SubgroupAccessControlHandler implements AccessControlInterface {

  use AccessControlTrait;

  /**
   * Constructs a new SubgroupAccessControlHandler.
   *
   * @param \Drupal\group\Plugin\Group\RelationHandler\AccessControlInterface $parent
   *   The default access control handler.
   */
  public function __construct(AccessControlInterface $parent) {
    $this->parent = $parent;
  }

  /**
   * {@inheritdoc}
   */
  public function entityCreateAccess(GroupInterface $group, AccountInterface $account, $return_as_object = FALSE) {
    // Normally, the entity create permission requires the plugin to specify
    // entity_access, but this is an exception because we control access without
    // specifying entity_access. So we can simply check the permission here.
    $permission = $this->permissionProvider->getPermission('create', 'entity');
    return $this->combinedPermissionCheck($group, $account, $permission, $return_as_object);
  }

}
