<?php

namespace Drupal\Tests\subgroup\Unit;

use Drupal\Core\Cache\Context\CacheContextsManager;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\group\Entity\GroupInterface;
use Drupal\group\Plugin\Group\Relation\GroupRelationType;
use Drupal\group\Plugin\Group\Relation\GroupRelationTypeManagerInterface;
use Drupal\group\Plugin\Group\RelationHandler\AccessControlInterface;
use Drupal\group\Plugin\Group\RelationHandler\PermissionProviderInterface;
use Drupal\subgroup\Plugin\Group\RelationHandler\SubgroupAccessControlHandler;
use Drupal\Tests\UnitTestCase;
use Prophecy\Argument;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Tests the Subgroup access control handler.
 *
 * @coversDefaultClass \Drupal\subgroup\Plugin\Group\RelationHandler\SubgroupAccessControlHandler
 * @group subgroup
 */
class SubgroupAccessControlHandlerTest extends UnitTestCase {

  /**
   * Tests the overrides to the parent access control handler.
   *
   * @covers ::entityCreateAccess
   * @dataProvider entityCreateAccessProvider
   */
  public function testEntityCreateAccess($expected, $has_permission) {
    $plugin_id = 'subgroup:foo';
    $entity_type_id = 'some_entity';
    $permission = 'some permission';

    $cache_context_manager = $this->prophesize(CacheContextsManager::class);
    $cache_context_manager->assertValidTokens(Argument::any())->willReturn(TRUE);

    $entity_type = $this->prophesize(EntityTypeInterface::class);
    $entity_type->entityClassImplements(Argument::any())->willReturn(FALSE);

    $entity_type_manager = $this->prophesize(EntityTypeManagerInterface::class);
    $entity_type_manager->getDefinition($entity_type_id)->willReturn($entity_type->reveal());

    $permission_provider = $this->prophesize(PermissionProviderInterface::class);
    $permission_provider->getPermission('create', 'entity')->willReturn($permission);
    $permission_provider->getAdminPermission()->willReturn(FALSE);

    $group_relation_type_manager = $this->prophesize(GroupRelationTypeManagerInterface::class);
    $group_relation_type_manager->getPermissionProvider($plugin_id)->willReturn($permission_provider->reveal());

    $container = $this->prophesize(ContainerInterface::class);
    $container->get('entity_type.manager')->willReturn($entity_type_manager->reveal());
    $container->get('group_relation_type.manager')->willReturn($group_relation_type_manager->reveal());
    $container->get('cache_contexts_manager')->willReturn($cache_context_manager->reveal());
    \Drupal::setContainer($container->reveal());

    $parent = $this->prophesize(AccessControlInterface::class);
    $account = $this->prophesize(AccountInterface::class)->reveal();

    $access_control_handler = new SubgroupAccessControlHandler($parent->reveal());
    $access_control_handler->init($plugin_id, new GroupRelationType([
      'entity_type_id' => $entity_type_id,
      'entity_access' => TRUE,
    ]));

    $group = $this->prophesize(GroupInterface::class);
    $group->hasPermission($permission, $account)->willReturn($has_permission);

    $this->assertSame($expected, $access_control_handler->entityCreateAccess($group->reveal(), $account));
  }

  /**
   * Data provider for testEntityCreateAccess().
   *
   * @return array
   *   A list of testEntityCreateAccess method arguments.
   */
  public function entityCreateAccessProvider() {
    $cases = [];

    $cases['has-permission'] = [
      'expected' => TRUE,
      'has_permission' => TRUE,
    ];

    $cases['no-has-permission'] = [
      'expected' => FALSE,
      'has_permission' => FALSE,
    ];

    return $cases;
  }

}
